@@ -4,31 +4,35 @@ from os import SEEK_SET
 from typing import List
 # import this to show warnings
 import warnings
+from matplotlib import pyplot as plt
 from tqdm import tqdm
 
 # channel definition
-from apread.channel import Channel
+from apread.entries import Channel, Group
 # binary reader to read binary files
 from apread.binaryReader import BinaryReader
 
 class APReader:
+    
     """
-    A reader which can read catmanAP binary files.
-
-    Instructions:
-        Do not change the order of readings in "read(self)"! This is crucial since the 
-        input file is formatted as binary.
-
-    Members:
-    ---------
-        filepath        The path to the .bin file.
-        Channels        List with Channels (channels.py).
+    All channels unsorted.    
     """
     Channels: List[Channel]
+    """
+    Grouped channels.    
+    """
+    Groups: List[Group]
 
     def __init__(self, path):
+        """Creates a new APReader based on a .binary file (path).
+
+        Args:
+            path (str): path to a catmanAP binary file.
+        """
         self.filepath = path
+        self.fileName = os.path.splitext(os.path.basename(path))[0]
         self.Channels = []
+        self.Groups = []
         self.read()
         self.connect()
 
@@ -68,7 +72,7 @@ class APReader:
             timeChannel = None            
             for channel in group:
                 # condition: channel name has to contain "Zeit"
-                if str.upper("Zeit") in str.upper(channel.name) or str.upper("Time") in str.upper(channel.name):
+                if str.upper("Zeit") in str.upper(channel.Name) or str.upper("Time") in str.upper(channel.Name):
                     timeChannel = channel
                     # there is only one time-channel
                     break
@@ -83,10 +87,34 @@ class APReader:
                         channel.isTime = True
             else:
                 print("\t [WARNING] Channel-group does not contain a time-channel")            
-        
+
+            # create new group based on the groups listed
+            self.Groups.append(Group(group, self.fileName))
         pass
 
-        
+    def __iter__(self):
+        """Iterates over everything in this reader.
+
+        Yields:
+            Any(Channel, Group): Yields all channels and groups.
+        """
+        for channel in self.Channels:
+            yield channel
+        for group in self.Groups:
+            yield group
+
+    def save(self, mode, path = None):
+        """Save reader as text.
+
+        Args:
+            mode (str): 'csv' or 'json'
+            path (str): the destination directory(!) path
+        """
+        if path == None:
+            path = os.path.dirname(self.filepath)
+
+        for thing in self:
+            thing.save(mode,path)
 
     def read(self):
         """
@@ -112,7 +140,7 @@ class APReader:
 
             # total number of channels
             self.numChannels = reader.read_int16()
-            print(f"\t[ APREAD ] Found {self.numChannels} Channels in {os.path.basename(self.filepath)}.")
+            print(f"\t[ {self.fileName} ] Found {self.numChannels} Channels.")
             # maximum channel length (usually 0 meaning unlimited)
             self.maxLength = reader.read_int32()
 
@@ -127,23 +155,36 @@ class APReader:
             for i in range(self.numChannels):
                 # create new channel on top of reader
                 # be careful with current stream position
-                channel = Channel(reader)
+                channel = Channel(reader, self.fileName)
 
                 if not channel.broken and channel.length > 0:                    
                     self.Channels.append(channel)
                 else:
-                    print(f'\t[ APREAD ] Skipping channel (zero length or invalid data)')
+                    print(f'\t[ {self.fileName} ] Skipping channel (zero length or invalid data)')
 
             # seek stream pointer to start of data
             reader.seek(self.dataOffset, SEEK_SET)
 
 
-            print('\t[ APREAD ] Reading Channels...')
+            print(f'\t[ {self.fileName} ] Reading Channels...')
             # loop through channels again and access data one after another
             for channel in tqdm(self.Channels, leave=False):
                 channel.readData()
 
-            print(f'\t[ APREAD ] Done. {len(self.Channels)} Channels left after filtering.')
+            print(f'\t[ {self.fileName} ] Done. {len(self.Channels)} Channels left after filtering.')
 
                     
             
+    def plot(self):
+        """Plots the complete file.
+        """
+        name = os.path.basename(self.filepath)
+        fig = plt.figure(name)
+
+        for group in self.Groups:
+            group.plot(governed=True)
+
+        plt.draw()
+        plt.title(name)
+        plt.legend()
+        plt.show()
\ No newline at end of file
